@@ -15,7 +15,6 @@
 #include "janus.h"
 #include "debug.h"
 #include "dtls.h"
-#include "rtp.h"
 #include "rtcp.h"
 #include "events.h"
 
@@ -106,7 +105,7 @@ void *janus_dtls_sctp_setup_thread(void *data);
 #endif
 
 
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if JANUS_USE_OPENSSL_PRE_1_1_API
 /*
  * DTLS locking stuff to make OpenSSL thread safe (not needed for 1.1.0)
  *
@@ -309,9 +308,9 @@ error:
 /* DTLS-SRTP initialization */
 gint janus_dtls_srtp_init(const char* server_pem, const char* server_key) {
 	const char *crypto_lib = NULL;
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#if JANUS_USE_OPENSSL_PRE_1_1_API
 #if defined(LIBRESSL_VERSION_NUMBER)
-	crypt_lib = "LibreSSL"
+	crypto_lib = "LibreSSL";
 #else
 	crypto_lib = "OpenSSL pre-1.1.0";
 #endif
@@ -332,7 +331,7 @@ gint janus_dtls_srtp_init(const char* server_pem, const char* server_key) {
 	JANUS_LOG(LOG_INFO, "Crypto: %s\n", crypto_lib);
 
 	/* Go on and create the DTLS context */
-#if OPENSSL_VERSION_NUMBER < 0x10100000L || defined(LIBRESSL_VERSION_NUMBER)
+#if JANUS_USE_OPENSSL_PRE_1_1_API
 	ssl_ctx = SSL_CTX_new(DTLSv1_method());
 #else
 	ssl_ctx = SSL_CTX_new(DTLS_method());
@@ -873,8 +872,8 @@ void janus_dtls_fd_bridge(janus_dtls_srtp *dtls) {
 		/* Update stats (TODO Do the same for the last second window as well)
 		 * FIXME: the Data stats includes the bytes used for the handshake */
 		if(bytes > 0) {
-			component->out_stats.data_packets++;
-			component->out_stats.data_bytes += bytes;
+			component->out_stats.data.packets++;
+			component->out_stats.data.bytes += bytes;
 		}
 		/* Check if there's anything left to send (e.g., fragmented packets) */
 		pending = BIO_ctrl_pending(dtls->filter_bio);
