@@ -143,11 +143,18 @@ $(document).ready(function() {
 								webrtcState: function(on) {
 									Janus.log("Janus says our WebRTC PeerConnection is " + (on ? "up" : "down") + " now");
 									$("#screencapture").parent().unblock();
-									bootbox.alert("Your screen sharing session just started: pass the <b>" + room + "</b> session identifier to those who want to attend.");
+									if(on) {
+										bootbox.alert("Your screen sharing session just started: pass the <b>" + room + "</b> session identifier to those who want to attend.");
+									} else {
+										bootbox.alert("Your screen sharing session just stopped.", function() {
+											janus.destroy();
+											window.location.reload();
+										});
+									}
 								},
 								onmessage: function(msg, jsep) {
 									Janus.debug(" ::: Got a message (publisher) :::");
-									Janus.debug(JSON.stringify(msg));
+									Janus.debug(msg);
 									var event = msg["videoroom"];
 									Janus.debug("Event: " + event);
 									if(event != undefined && event != null) {
@@ -221,7 +228,7 @@ $(document).ready(function() {
 								},
 								onlocalstream: function(stream) {
 									Janus.debug(" ::: Got a local stream :::");
-									Janus.debug(JSON.stringify(stream));
+									Janus.debug(stream);
 									$('#screenmenu').hide();
 									$('#room').removeClass('hide').show();
 									if($('#screenvideo').length === 0) {
@@ -236,6 +243,11 @@ $(document).ready(function() {
 											color: 'white'
 										}
 									});
+									// Track the onended event on the video track, just in case the user
+									// stops screensharing without closing the PeerConnection too
+									screentest.webrtcStuff.myStream.getVideoTracks()[0].onended = function() {
+										screentest.hangup();
+									}
 								},
 								onremotestream: function(stream) {
 									// The publisher stream is sendonly, we don't expect anything here
@@ -412,7 +424,7 @@ function newRemoteFeed(id, display) {
 			},
 			onmessage: function(msg, jsep) {
 				Janus.debug(" ::: Got a message (listener) :::");
-				Janus.debug(JSON.stringify(msg));
+				Janus.debug(msg);
 				var event = msg["videoroom"];
 				Janus.debug("Event: " + event);
 				if(event != undefined && event != null) {
@@ -456,11 +468,13 @@ function newRemoteFeed(id, display) {
 				// The subscriber stream is recvonly, we don't expect anything here
 			},
 			onremotestream: function(stream) {
-				if($('#screenvideo').length === 0) {
-					// No remote video yet
-					$('#screencapture').append('<video class="rounded centered" id="waitingvideo" width="100%" height="100%" />');
-					$('#screencapture').append('<video class="rounded centered hide" id="screenvideo" width="100%" height="100%" autoplay/>');
+				if($('#screenvideo').length > 0) {
+					// Been here already
+					return;
 				}
+				// No remote video yet
+				$('#screencapture').append('<video class="rounded centered" id="waitingvideo" width="100%" height="100%" />');
+				$('#screencapture').append('<video class="rounded centered hide" id="screenvideo" width="100%" height="100%" autoplay/>');
 				// Show the video, hide the spinner and show the resolution when we get a playing event
 				$("#screenvideo").bind("playing", function () {
 					$('#waitingvideo').remove();
