@@ -24,6 +24,7 @@
 #include "dtls.h"
 #include "sctp.h"
 #include "rtcp.h"
+#include "text2pcap.h"
 #include "utils.h"
 #include "plugins/plugin.h"
 
@@ -176,42 +177,40 @@ typedef struct janus_ice_trickle janus_ice_trickle;
 #define JANUS_ICE_HANDLE_WEBRTC_ALL_TRICKLES		(1 << 8)
 #define JANUS_ICE_HANDLE_WEBRTC_TRICKLE_SYNCED		(1 << 9)
 #define JANUS_ICE_HANDLE_WEBRTC_DATA_CHANNELS		(1 << 10)
-#define JANUS_ICE_HANDLE_WEBRTC_PLAN_B				(1 << 11)
-#define JANUS_ICE_HANDLE_WEBRTC_CLEANING			(1 << 12)
-#define JANUS_ICE_HANDLE_WEBRTC_HAS_AUDIO			(1 << 13)
-#define JANUS_ICE_HANDLE_WEBRTC_HAS_VIDEO			(1 << 14)
-#define JANUS_ICE_HANDLE_WEBRTC_GOT_OFFER			(1 << 15)
-#define JANUS_ICE_HANDLE_WEBRTC_GOT_ANSWER			(1 << 16)
-#define JANUS_ICE_HANDLE_WEBRTC_HAS_AGENT			(1 << 17)
+#define JANUS_ICE_HANDLE_WEBRTC_CLEANING			(1 << 11)
+#define JANUS_ICE_HANDLE_WEBRTC_HAS_AUDIO			(1 << 12)
+#define JANUS_ICE_HANDLE_WEBRTC_HAS_VIDEO			(1 << 13)
+#define JANUS_ICE_HANDLE_WEBRTC_GOT_OFFER			(1 << 14)
+#define JANUS_ICE_HANDLE_WEBRTC_GOT_ANSWER			(1 << 15)
+#define JANUS_ICE_HANDLE_WEBRTC_HAS_AGENT			(1 << 16)
 
 
 /*! \brief Janus media statistics
  * \note To improve with more stuff */
+typedef struct janus_ice_stats_info {
+	/*! \brief Packets sent or received */
+	guint32 packets;
+	/*! \brief Bytes sent or received */
+	guint64 bytes;
+	/*! \brief Bytes sent or received in the last second */
+	guint32 bytes_lastsec, bytes_lastsec_temp;
+	/*! \brief Time we last updated the last second counter */
+	gint64 updated;
+	/*! \brief Whether or not we notified about lastsec issues already */
+	gboolean notified_lastsec;
+	/*! \brief Number of NACKs sent or received */
+	guint32 nacks;
+} janus_ice_stats_info;
+
+/*! \brief Janus media statistics container
+ * \note To improve with more stuff */
 typedef struct janus_ice_stats {
-	/*! \brief Audio packets sent or received */
-	guint32 audio_packets;
-	/*! \brief Audio bytes sent or received */
-	guint64 audio_bytes;
-	/*! \brief Audio bytes sent or received in the last second */
-	GList *audio_bytes_lastsec;
-	/*! \brief Whether or not we notified about audio lastsec issues already */
-	gboolean audio_notified_lastsec;
-	/*! \brief Number of audio NACKs sent or received */
-	guint32 audio_nacks;
-	/*! \brief Video packets sent or received */
-	guint32 video_packets;
-	/*! \brief Video bytes sent or received */
-	guint64 video_bytes;
-	/*! \brief Video bytes sent or received in the last second */
-	GList *video_bytes_lastsec;
-	/*! \brief Whether or not we notified about video lastsec issues already */
-	gboolean video_notified_lastsec;
-	/*! \brief Number of video NACKs sent or received */
-	guint32 video_nacks;
-	/*! \brief Data packets sent or received */
-	guint32 data_packets;
-	/*! \brief Data bytes sent or received */
-	guint64 data_bytes;
+	/*! \brief Audio info */
+	janus_ice_stats_info audio;
+	/*! \brief Video info (considering we may be simulcasting) */
+	janus_ice_stats_info video[3];
+	/*! \brief Data info */
+	janus_ice_stats_info data;
 	/*! \brief Last time the slow_link callback (of the plugin) was called */
 	gint64 last_slowlink_time;
 	/*! \brief Start time of recent NACKs (for slow_link) */
@@ -220,19 +219,6 @@ typedef struct janus_ice_stats {
 	guint sl_nack_recent_cnt;
 } janus_ice_stats;
 
-/*! \brief Janus media statistics: received packet info
- * \note To improve with more stuff */
-typedef struct janus_ice_stats_item {
-	/*! \brief Bytes sent or received */
-	guint64 bytes;
-	/*! \brief Time at which this happened */
-	gint64 when;
-} janus_ice_stats_item;
-
-/*! \brief Quick helper method to reset stats
- * @param stats The janus_ice_stats instance to reset */
-void janus_ice_stats_reset(janus_ice_stats *stats);
-
 /*! \brief Quick helper method to notify a WebRTC hangup through the Janus API
  * @param handle The janus_ice_handle instance this event refers to
  * @param reason A description of why this happened */
@@ -323,6 +309,8 @@ struct janus_ice_handle {
 	gchar *local_sdp;
 	/*! \brief SDP received by the peer (just for debugging purposes) */
 	gchar *remote_sdp;
+	/*! \brief Reason this handle has been hung up*/
+	const gchar *hangup_reason;
 	/*! \brief List of pending trickle candidates (those we received before getting the JSEP offer) */
 	GList *pending_trickles;
 	/*! \brief Queue of outgoing packets to send */
@@ -335,6 +323,10 @@ struct janus_ice_handle {
 	guint srtp_errors_count;
 	/*! \brief Count of the recent SRTP replay errors, in order to avoid spamming the logs */
 	gint last_srtp_error;
+	/*! \brief Flag to decide whether or not packets need to be dumped to a text2pcap file */
+	volatile gint dump_packets;
+	/*! \brief In case this session must be saved to text2pcap, the instance to dump packets to */
+	janus_text2pcap *text2pcap;
 	/*! \brief Mutex to lock/unlock the ICE session */
 	janus_mutex mutex;
 };
@@ -355,10 +347,12 @@ struct janus_ice_stream {
 	guint32 video_ssrc;
 	/*! \brief Audio SSRC of the peer for this stream (may be bundled) */
 	guint32 audio_ssrc_peer;
-	/*! \brief Video SSRC of the peer for this stream (may be bundled) */
-	guint32 video_ssrc_peer;
+	/*! \brief Video SSRC(s) of the peer for this stream (may be bundled, and simulcasting) */
+	guint32 video_ssrc_peer[3];
 	/*! \brief Video retransmissions SSRC of the peer for this stream (may be bundled) */
 	guint32 video_ssrc_peer_rtx;
+	/*! \brief Array of RTP Stream IDs (for Firefox simulcasting, if enabled) */
+	char *rid[3];
 	/*! \brief List of payload types we can expect for audio */
 	GList *audio_payload_types;
 	/*! \brief List of payload types we can expect for video */
@@ -367,16 +361,16 @@ struct janus_ice_stream {
 	gint payload_type;
 	/*! \brief RTCP context for the audio stream (may be bundled) */
 	rtcp_context *audio_rtcp_ctx;
-	/*! \brief RTCP context for the video stream (may be bundled) */
-	rtcp_context *video_rtcp_ctx;
+	/*! \brief RTCP context(s) for the video stream (may be bundled, and simulcasting) */
+	rtcp_context *video_rtcp_ctx[3];
 	/*! \brief First received audio NTP timestamp */
 	gint64 audio_first_ntp_ts;
 	/*! \brief First received audio RTP timestamp */
 	guint32 audio_first_rtp_ts;
-	/*! \brief First received video NTP timestamp */
-	gint64 video_first_ntp_ts;
-	/*! \brief First received video NTP RTP timestamp */
-	guint32 video_first_rtp_ts;
+	/*! \brief First received video NTP timestamp (for all simulcast video streams) */
+	gint64 video_first_ntp_ts[3];
+	/*! \brief First received video NTP RTP timestamp (for all simulcast video streams) */
+	guint32 video_first_rtp_ts[3];
 	/*! \brief Last sent audio RTP timestamp */
 	guint32 audio_last_ts;
 	/*! \brief Last sent video RTP timestamp */
@@ -434,8 +428,14 @@ struct janus_ice_component {
 	GSource *dtlsrt_source;
 	/*! \brief DTLS-SRTP stack */
 	janus_dtls_srtp *dtls;
+	/*! \brief Whether we should do NACKs (in or out) for audio */
+	gboolean do_audio_nacks;
+	/*! \brief Whether we should do NACKs (in or out) for video */
+	gboolean do_video_nacks;
 	/*! \brief List of previously sent janus_rtp_packet RTP packets, in case we receive NACKs */
-	GList *retransmit_buffer;
+	GQueue *retransmit_buffer;
+	/*! \brief HashTable of retransmittable sequence numbers, in case we receive NACKs */
+	GHashTable *retransmit_seqs;
 	/*! \brief Last time a log message about sending retransmits was printed */
 	gint64 retransmit_log_ts;
 	/*! \brief Number of retransmitted packets since last log message */
@@ -446,8 +446,8 @@ struct janus_ice_component {
 	guint nack_sent_recent_cnt;
 	/*! \brief List of recently received audio sequence numbers (as a support to NACK generation) */
 	janus_seq_info *last_seqs_audio;
-	/*! \brief List of recently received video sequence numbers (as a support to NACK generation) */
-	janus_seq_info *last_seqs_video;
+	/*! \brief List of recently received video sequence numbers (as a support to NACK generation, for each simulcast SSRC) */
+	janus_seq_info *last_seqs_video[3];
 	/*! \brief Stats for incoming data (audio/video/data) */
 	janus_ice_stats in_stats;
 	/*! \brief Stats for outgoing data (audio/video/data) */
