@@ -263,8 +263,8 @@
 
 
 /* Plugin information */
-#define JANUS_RECORDPLAY_VERSION			3
-#define JANUS_RECORDPLAY_VERSION_STRING		"0.0.3"
+#define JANUS_RECORDPLAY_VERSION			4
+#define JANUS_RECORDPLAY_VERSION_STRING		"0.0.4"
 #define JANUS_RECORDPLAY_DESCRIPTION		"This is a trivial Record&Play plugin for Janus, to record WebRTC sessions and replay them."
 #define JANUS_RECORDPLAY_NAME				"JANUS Record&Play plugin"
 #define JANUS_RECORDPLAY_AUTHOR				"Meetecho s.r.l."
@@ -347,6 +347,7 @@ static janus_callbacks *gateway = NULL;
 static GThread *handler_thread;
 static GThread *watchdog;
 static void *janus_recordplay_handler(void *data);
+static void janus_recordplay_hangup_media_internal(janus_plugin_session *handle);
 
 typedef struct janus_recordplay_message {
 	janus_plugin_session *handle;
@@ -377,7 +378,11 @@ typedef struct janus_recordplay_recording {
 	char *name;			/* Name of the recording */
 	char *date;			/* Time of the recording */
 	char *arc_file;		/* Audio file name */
+	const char *acodec;	/* Codec used for audio, if available */
+	int audio_pt;		/* Payload types to use for audio when playing recordings */
 	char *vrc_file;		/* Video file name */
+	const char *vcodec;	/* Codec used for video, if available */
+	int video_pt;		/* Payload types to use for audio when playing recordings */
 	gboolean completed;	/* Whether this recording was completed or still going on */
 	char *offer;		/* The SDP offer that will be sent to watchers */
 	GList *viewers;		/* List of users watching this recording */
@@ -385,13 +390,13 @@ typedef struct janus_recordplay_recording {
 	janus_mutex mutex;	/* Mutex for this recording */
 } janus_recordplay_recording;
 static GHashTable *recordings = NULL;
-static janus_mutex recordings_mutex;
+static janus_mutex recordings_mutex = JANUS_MUTEX_INITIALIZER;
 
 typedef struct janus_recordplay_session {
 	janus_plugin_session *handle;
 	gboolean active;
 	gboolean recorder;		/* Whether this session is used to record or to replay a WebRTC session */
-	gboolean firefox;	/* We send Firefox users a different kind of FIR */
+	gboolean firefox;		/* We send Firefox users a different kind of FIR */
 	janus_recordplay_recording *recording;
 	janus_recorder *arc;	/* Audio recorder */
 	janus_recorder *vrc;	/* Video recorder */
@@ -404,12 +409,13 @@ typedef struct janus_recordplay_session {
 	guint video_keyframe_interval; /* keyframe request interval (ms) */
 	guint64 video_keyframe_request_last; /* timestamp of last keyframe request sent */
 	gint video_fir_seq;
+	guint32 simulcast_ssrc;	/* We don't support Simulcast in this plugin, so we'll stick to the base substream */
 	volatile gint hangingup;
 	gint64 destroyed;	/* Time at which this session was marked as destroyed */
 } janus_recordplay_session;
 static GHashTable *sessions;
 static GList *old_sessions;
-static janus_mutex sessions_mutex;
+static janus_mutex sessions_mutex = JANUS_MUTEX_INITIALIZER;
 
 
 static char *recordings_path = NULL;
@@ -419,28 +425,162 @@ static void *janus_recordplay_playout_thread(void *data);
 /* Helper to send RTCP feedback back to recorders, if needed */
 void janus_recordplay_send_rtcp_feedback(janus_plugin_session *handle, int video, char *buf, int len);
 
-/* To make things easier, we use static payload types for viewers */
-#define OPUS_PT		111
-#define VP8_PT		100
+/* To make things easier, we use static payload types for viewers (unless it's for G.711 or G.722) */
+#define AUDIO_PT		111
+#define VIDEO_PT		100
+
+/* Helper method to check which codec was used in a specific recording */
+static const char *janus_recordplay_parse_codec(const char *dir, const char *filename) {
+	if(dir == NULL || filename == NULL)
+		return NULL;
+	char source[1024];
+	if(strstr(filename, ".mjr"))
+		g_snprintf(source, 1024, "%s/%s", dir, filename);
+	else
+		g_snprintf(source, 1024, "%s/%s.mjr", dir, filename);
+	FILE *file = fopen(source, "rb");
+	if(file == NULL) {
+		JANUS_LOG(LOG_ERR, "Could not open file %s\n", source);
+		return NULL;
+	}
+	fseek(file, 0L, SEEK_END);
+	long fsize = ftell(file);
+	fseek(file, 0L, SEEK_SET);
+
+	/* Pre-parse */
+	JANUS_LOG(LOG_VERB, "Pre-parsing file %s to generate ordered index...\n", source);
+	gboolean parsed_header = FALSE;
+	int bytes = 0;
+	long offset = 0;
+	uint16_t len = 0;
+	char prebuffer[1500];
+	memset(prebuffer, 0, 1500);
+	/* Let's look for timestamp resets first */
+	while(offset < fsize) {
+		/* Read frame header */
+		fseek(file, offset, SEEK_SET);
+		bytes = fread(prebuffer, sizeof(char), 8, file);
+		if(bytes != 8 || prebuffer[0] != 'M') {
+			JANUS_LOG(LOG_ERR, "Invalid header...\n");
+			fclose(file);
+			return NULL;
+		}
+		if(prebuffer[1] == 'E') {
+			/* Either the old .mjr format header ('MEETECHO' header followed by 'audio' or 'video'), or a frame */
+			offset += 8;
+			bytes = fread(&len, sizeof(uint16_t), 1, file);
+			len = ntohs(len);
+			offset += 2;
+			if(len == 5 && !parsed_header) {
+				/* This is the main header */
+				parsed_header = TRUE;
+				bytes = fread(prebuffer, sizeof(char), 5, file);
+				if(prebuffer[0] == 'v') {
+					JANUS_LOG(LOG_VERB, "This is an old video recording, assuming VP8\n");
+					fclose(file);
+					return "vp8";
+				} else if(prebuffer[0] == 'a') {
+					JANUS_LOG(LOG_VERB, "This is an old audio recording, assuming Opus\n");
+					fclose(file);
+					return "opus";
+				}
+			}
+			JANUS_LOG(LOG_WARN, "Unsupported recording media type...\n");
+			fclose(file);
+			return NULL;
+		} else if(prebuffer[1] == 'J') {
+			/* New .mjr format */
+			offset += 8;
+			bytes = fread(&len, sizeof(uint16_t), 1, file);
+			len = ntohs(len);
+			offset += 2;
+			if(len > 0 && !parsed_header) {
+				/* This is the info header */
+				bytes = fread(prebuffer, sizeof(char), len, file);
+				if(bytes < 0) {
+					JANUS_LOG(LOG_ERR, "Error reading from file... %s\n", strerror(errno));
+					fclose(file);
+					return NULL;
+				}
+				parsed_header = TRUE;
+				prebuffer[len] = '\0';
+				json_error_t error;
+				json_t *info = json_loads(prebuffer, 0, &error);
+				if(!info) {
+					JANUS_LOG(LOG_ERR, "JSON error: on line %d: %s\n", error.line, error.text);
+					JANUS_LOG(LOG_WARN, "Error parsing info header...\n");
+					fclose(file);
+					return NULL;
+				}
+				/* Is it audio or video? */
+				json_t *type = json_object_get(info, "t");
+				if(!type || !json_is_string(type)) {
+					JANUS_LOG(LOG_WARN, "Missing/invalid recording type in info header...\n");
+					json_decref(info);
+					fclose(file);
+					return NULL;
+				}
+				const char *t = json_string_value(type);
+				gboolean video = FALSE;
+				if(!strcasecmp(t, "v")) {
+					video = TRUE;
+				} else if(!strcasecmp(t, "a")) {
+					video = FALSE;
+				} else {
+					JANUS_LOG(LOG_WARN, "Unsupported recording type '%s' in info header...\n", t);
+					json_decref(info);
+					fclose(file);
+					return NULL;
+				}
+				/* What codec was used? */
+				json_t *codec = json_object_get(info, "c");
+				if(!codec || !json_is_string(codec)) {
+					JANUS_LOG(LOG_WARN, "Missing recording codec in info header...\n");
+					json_decref(info);
+					fclose(file);
+					return NULL;
+				}
+				const char *c = json_string_value(codec);
+				const char *mcodec = janus_sdp_match_preferred_codec(video ? JANUS_SDP_VIDEO : JANUS_SDP_AUDIO, (char *)c);
+				if(mcodec != NULL) {
+					/* Found! */
+					json_decref(info);
+					fclose(file);
+					return mcodec;
+				}
+				json_decref(info);
+			}
+			JANUS_LOG(LOG_WARN, "No codec found...\n");
+			fclose(file);
+			return NULL;
+		} else {
+			JANUS_LOG(LOG_ERR, "Invalid header...\n");
+			fclose(file);
+			return NULL;
+		}
+	}
+	fclose(file);
+	return NULL;
+}
 
 /* Helper method to prepare an SDP offer when a recording is available */
 static int janus_recordplay_generate_offer(janus_recordplay_recording *rec) {
 	if(rec == NULL)
 		return -1;
 	/* Prepare an SDP offer we'll send to playout viewers */
-	gboolean offer_audio = (rec->arc_file != NULL),
-		offer_video = (rec->vrc_file != NULL);
+	gboolean offer_audio = (rec->arc_file != NULL && rec->acodec != NULL),
+		offer_video = (rec->vrc_file != NULL && rec->vcodec != NULL);
 	char s_name[100];
 	g_snprintf(s_name, sizeof(s_name), "Recording %"SCNu64, rec->id);
 	janus_sdp *offer = janus_sdp_generate_offer(
 		s_name, "1.1.1.1",
 		JANUS_SDP_OA_AUDIO, offer_audio,
-		JANUS_SDP_OA_AUDIO_CODEC, "opus",
-		JANUS_SDP_OA_AUDIO_PT, OPUS_PT,
+		JANUS_SDP_OA_AUDIO_CODEC, rec->acodec,
+		JANUS_SDP_OA_AUDIO_PT, rec->audio_pt,
 		JANUS_SDP_OA_AUDIO_DIRECTION, JANUS_SDP_SENDONLY,
 		JANUS_SDP_OA_VIDEO, offer_video,
-		JANUS_SDP_OA_VIDEO_CODEC, "vp8",
-		JANUS_SDP_OA_VIDEO_PT, VP8_PT,
+		JANUS_SDP_OA_VIDEO_CODEC, rec->vcodec,
+		JANUS_SDP_OA_VIDEO_PT, rec->video_pt,
 		JANUS_SDP_OA_VIDEO_DIRECTION, JANUS_SDP_SENDONLY,
 		JANUS_SDP_OA_DATA, FALSE,
 		JANUS_SDP_OA_DONE);
@@ -569,11 +709,9 @@ int janus_recordplay_init(janus_callbacks *callback, const char *config_path) {
 		}
 	}
 	recordings = g_hash_table_new_full(g_int64_hash, g_int64_equal, (GDestroyNotify)g_free, NULL);
-	janus_mutex_init(&recordings_mutex);
 	janus_recordplay_update_recordings_list();
 	
 	sessions = g_hash_table_new(NULL, NULL);
-	janus_mutex_init(&sessions_mutex);
 	messages = g_async_queue_new_full((GDestroyNotify) janus_recordplay_message_free);
 	/* This is the callback we'll need to invoke to contact the gateway */
 	gateway = callback;
@@ -654,6 +792,14 @@ const char *janus_recordplay_get_package(void) {
 	return JANUS_RECORDPLAY_PACKAGE;
 }
 
+static janus_recordplay_session *janus_recordplay_lookup_session(janus_plugin_session *handle) {
+	janus_recordplay_session *session = NULL;
+	if (g_hash_table_contains(sessions, handle)) {
+		session = (janus_recordplay_session *)handle->plugin_handle;
+	}
+	return session;
+}
+
 void janus_recordplay_create_session(janus_plugin_session *handle, int *error) {
 	if(g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized)) {
 		*error = -1;
@@ -688,16 +834,17 @@ void janus_recordplay_destroy_session(janus_plugin_session *handle, int *error)
 		*error = -1;
 		return;
 	}	
-	janus_recordplay_session *session = (janus_recordplay_session *)handle->plugin_handle;
+	janus_mutex_lock(&sessions_mutex);
+	janus_recordplay_session *session = janus_recordplay_lookup_session(handle);
 	if(!session) {
+		janus_mutex_unlock(&sessions_mutex);
 		JANUS_LOG(LOG_ERR, "No Record&Play session associated with this handle...\n");
 		*error = -2;
 		return;
 	}
-	janus_mutex_lock(&sessions_mutex);
 	if(!session->destroyed) {
 		JANUS_LOG(LOG_VERB, "Removing Record&Play session...\n");
-		janus_recordplay_hangup_media(handle);
+		janus_recordplay_hangup_media_internal(handle);
 		session->destroyed = janus_get_monotonic_time();
 		g_hash_table_remove(sessions, handle);
 		/* Cleaning up and removing the session is done in a lazy way */
@@ -711,8 +858,10 @@ json_t *janus_recordplay_query_session(janus_plugin_session *handle) {
 	if(g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized)) {
 		return NULL;
 	}	
-	janus_recordplay_session *session = (janus_recordplay_session *)handle->plugin_handle;
+	janus_mutex_lock(&sessions_mutex);
+	janus_recordplay_session *session = janus_recordplay_lookup_session(handle);
 	if(!session) {
+		janus_mutex_unlock(&sessions_mutex);
 		JANUS_LOG(LOG_ERR, "No session associated with this handle...\n");
 		return NULL;
 	}
@@ -724,6 +873,7 @@ json_t *janus_recordplay_query_session(janus_plugin_session *handle) {
 		json_object_set_new(info, "recording_name", json_string(session->recording->name));
 	}
 	json_object_set_new(info, "destroyed", json_integer(session->destroyed));
+	janus_mutex_unlock(&sessions_mutex);
 	return info;
 }
 
@@ -737,6 +887,8 @@ struct janus_plugin_result *janus_recordplay_handle_message(janus_plugin_session
 	json_t *root = message;
 	json_t *response = NULL;
 	
+	janus_mutex_lock(&sessions_mutex);
+
 	if(message == NULL) {
 		JANUS_LOG(LOG_ERR, "No message??\n");
 		error_code = JANUS_RECORDPLAY_ERROR_NO_MESSAGE;
@@ -744,7 +896,7 @@ struct janus_plugin_result *janus_recordplay_handle_message(janus_plugin_session
 		goto plugin_response;
 	}
 
-	janus_recordplay_session *session = (janus_recordplay_session *)handle->plugin_handle;	
+	janus_recordplay_session *session = janus_recordplay_lookup_session(handle);
 	if(!session) {
 		JANUS_LOG(LOG_ERR, "No session associated with this handle...\n");
 		error_code = JANUS_RECORDPLAY_ERROR_UNKNOWN_ERROR;
@@ -796,7 +948,11 @@ struct janus_plugin_result *janus_recordplay_handle_message(janus_plugin_session
 			json_object_set_new(ml, "name", json_string(rec->name));
 			json_object_set_new(ml, "date", json_string(rec->date));
 			json_object_set_new(ml, "audio", rec->arc_file ? json_true() : json_false());
+			if(rec->acodec)
+				json_object_set_new(ml, "audio_codec", json_string(rec->acodec));
 			json_object_set_new(ml, "video", rec->vrc_file ? json_true() : json_false());
+			if(rec->vcodec)
+				json_object_set_new(ml, "video_codec", json_string(rec->vcodec));
 			json_array_append_new(list, ml);
 		}
 		janus_mutex_unlock(&recordings_mutex);
@@ -833,6 +989,7 @@ struct janus_plugin_result *janus_recordplay_handle_message(janus_plugin_session
 	} else if(!strcasecmp(request_text, "record") || !strcasecmp(request_text, "play")
 			|| !strcasecmp(request_text, "start") || !strcasecmp(request_text, "stop")) {
 		/* These messages are handled asynchronously */
+		janus_mutex_unlock(&sessions_mutex);
 		janus_recordplay_message *msg = g_malloc0(sizeof(janus_recordplay_message));
 		msg->handle = handle;
 		msg->transaction = transaction;
@@ -850,6 +1007,7 @@ struct janus_plugin_result *janus_recordplay_handle_message(janus_plugin_session
 
 plugin_response:
 		{
+			janus_mutex_unlock(&sessions_mutex);
 			if(error_code == 0 && !response) {
 				error_code = JANUS_RECORDPLAY_ERROR_UNKNOWN_ERROR;
 				g_snprintf(error_cause, 512, "Invalid response");
@@ -877,13 +1035,17 @@ void janus_recordplay_setup_media(janus_plugin_session *handle) {
 	JANUS_LOG(LOG_INFO, "WebRTC media is now available\n");
 	if(g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized))
 		return;
-	janus_recordplay_session *session = (janus_recordplay_session *)handle->plugin_handle;	
+	janus_mutex_lock(&sessions_mutex);
+	janus_recordplay_session *session = janus_recordplay_lookup_session(handle);
 	if(!session) {
+		janus_mutex_unlock(&sessions_mutex);
 		JANUS_LOG(LOG_ERR, "No session associated with this handle...\n");
 		return;
 	}
-	if(session->destroyed)
+	if(session->destroyed) {
+		janus_mutex_unlock(&sessions_mutex);
 		return;
+	}
 	g_atomic_int_set(&session->hangingup, 0);
 	/* Take note of the fact that the session is now active */
 	session->active = TRUE;
@@ -895,6 +1057,7 @@ void janus_recordplay_setup_media(janus_plugin_session *handle) {
 			JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the Record&Play playout thread...\n", error->code, error->message ? error->message : "??");
 		}
 	}
+	janus_mutex_unlock(&sessions_mutex);
 }
 
 void janus_recordplay_send_rtcp_feedback(janus_plugin_session *handle, int video, char *buf, int len) {
@@ -951,6 +1114,15 @@ void janus_recordplay_incoming_rtp(janus_plugin_session *handle, int video, char
 		}
 		if(session->destroyed)
 			return;
+		if(video && session->simulcast_ssrc) {
+			/* The user is simulcasting: drop everything except the base layer */
+			janus_rtp_header *header = (janus_rtp_header *)buf;
+			uint32_t ssrc = ntohl(header->ssrc);
+			if(ssrc != session->simulcast_ssrc) {
+				JANUS_LOG(LOG_DBG, "Dropping packet (not base simulcast substream)\n");
+				return;
+			}
+		}
 		/* Are we recording? */
 		if(session->recorder) {
 			janus_recorder_save_frame(video ? session->vrc : session->arc, buf, len);
@@ -975,9 +1147,12 @@ void janus_recordplay_slow_link(janus_plugin_session *handle, int uplink, int vi
 	if(handle == NULL || handle->stopped || g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized) || !gateway)
 		return;
 
-	janus_recordplay_session *session = (janus_recordplay_session *)handle->plugin_handle;
-	if(!session || session->destroyed)
+	janus_mutex_lock(&sessions_mutex);
+	janus_recordplay_session *session = janus_recordplay_lookup_session(handle);
+	if(!session || session->destroyed) {
+		janus_mutex_unlock(&sessions_mutex);
 		return;
+	}
 
 	json_t *event = json_object();
 	json_object_set_new(event, "recordplay", json_string("event"));
@@ -989,22 +1164,32 @@ void janus_recordplay_slow_link(janus_plugin_session *handle, int uplink, int vi
 	json_object_set_new(event, "result", result);
 	gateway->push_event(session->handle, &janus_recordplay_plugin, NULL, event, NULL);
 	json_decref(event);
+	janus_mutex_unlock(&sessions_mutex);
 }
 
 void janus_recordplay_hangup_media(janus_plugin_session *handle) {
+	janus_mutex_lock(&sessions_mutex);
+	janus_recordplay_hangup_media_internal(handle);
+	janus_mutex_unlock(&sessions_mutex);
+}
+
+static void janus_recordplay_hangup_media_internal(janus_plugin_session *handle) {
 	JANUS_LOG(LOG_INFO, "No WebRTC media anymore\n");
 	if(g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized))
 		return;
-	janus_recordplay_session *session = (janus_recordplay_session *)handle->plugin_handle;
+	janus_recordplay_session *session = janus_recordplay_lookup_session(handle);
 	if(!session) {
 		JANUS_LOG(LOG_ERR, "No session associated with this handle...\n");
 		return;
 	}
 	session->active = FALSE;
-	if(session->destroyed || !session->recorder)
+	if(session->destroyed || !session->recorder) {
 		return;
-	if(g_atomic_int_add(&session->hangingup, 1))
+	}
+	if(g_atomic_int_add(&session->hangingup, 1)) {
 		return;
+	}
+	session->simulcast_ssrc = 0;
 
 	/* Send an event to the browser and tell it's over */
 	json_t *event = json_object();
@@ -1014,13 +1199,67 @@ void janus_recordplay_hangup_media(janus_plugin_session *handle) {
 	JANUS_LOG(LOG_VERB, "  >> Pushing event: %d (%s)\n", ret, janus_get_api_error(ret));
 	json_decref(event);
 
-	/* FIXME Simulate a "stop" coming from the browser */
-	janus_recordplay_message *msg = g_malloc0(sizeof(janus_recordplay_message));
-	msg->handle = handle;
-	msg->message = json_pack("{ss}", "request", "stop");
-	msg->transaction = NULL;
-	msg->jsep = NULL;
-	g_async_queue_push(messages, msg);
+	session->active = FALSE;
+	janus_mutex_lock(&session->rec_mutex);
+	if(session->arc) {
+		janus_recorder_close(session->arc);
+		JANUS_LOG(LOG_INFO, "Closed audio recording %s\n", session->arc->filename ? session->arc->filename : "??");
+		janus_recorder_free(session->arc);
+	}
+	session->arc = NULL;
+	if(session->vrc) {
+		janus_recorder_close(session->vrc);
+		JANUS_LOG(LOG_INFO, "Closed video recording %s\n", session->vrc->filename ? session->vrc->filename : "??");
+		janus_recorder_free(session->vrc);
+	}
+	session->vrc = NULL;
+	janus_mutex_unlock(&session->rec_mutex);
+	if(session->recorder) {
+		/* Create a .nfo file for this recording */
+		char nfofile[1024], nfo[1024];
+		nfofile[0] = '\0';
+		nfo[0] = '\0';
+		g_snprintf(nfofile, 1024, "%s/%"SCNu64".nfo", recordings_path, session->recording->id);
+		FILE *file = fopen(nfofile, "wt");
+		if(file == NULL) {
+			JANUS_LOG(LOG_ERR, "Error creating file %s...\n", nfofile);
+		} else {
+			if(session->recording->arc_file && session->recording->vrc_file) {
+				g_snprintf(nfo, 1024,
+					"[%"SCNu64"]\r\n"
+					"name = %s\r\n"
+					"date = %s\r\n"
+					"audio = %s.mjr\r\n"
+					"video = %s.mjr\r\n",
+						session->recording->id, session->recording->name, session->recording->date,
+						session->recording->arc_file, session->recording->vrc_file);
+			} else if(session->recording->arc_file) {
+				g_snprintf(nfo, 1024,
+					"[%"SCNu64"]\r\n"
+					"name = %s\r\n"
+					"date = %s\r\n"
+					"audio = %s.mjr\r\n",
+						session->recording->id, session->recording->name, session->recording->date,
+						session->recording->arc_file);
+			} else if(session->recording->vrc_file) {
+				g_snprintf(nfo, 1024,
+					"[%"SCNu64"]\r\n"
+					"name = %s\r\n"
+					"date = %s\r\n"
+					"video = %s.mjr\r\n",
+						session->recording->id, session->recording->name, session->recording->date,
+						session->recording->vrc_file);
+			}
+			/* Write to the file now */
+			fwrite(nfo, strlen(nfo), sizeof(char), file);
+			fclose(file);
+			/* Generate the offer */
+			if(janus_recordplay_generate_offer(session->recording) < 0) {
+				JANUS_LOG(LOG_WARN, "Could not generate offer for recording %"SCNu64"...\n", session->recording->id);
+			}
+			session->recording->completed = TRUE;
+		}
+	}
 }
 
 /* Thread to handle incoming messages */
@@ -1040,11 +1279,8 @@ static void *janus_recordplay_handler(void *data) {
 			janus_recordplay_message_free(msg);
 			continue;
 		}
-		janus_recordplay_session *session = NULL;
 		janus_mutex_lock(&sessions_mutex);
-		if(g_hash_table_lookup(sessions, msg->handle) != NULL ) {
-			session = (janus_recordplay_session *)msg->handle->plugin_handle;
-		}
+		janus_recordplay_session *session = janus_recordplay_lookup_session(msg->handle);
 		if(!session) {
 			janus_mutex_unlock(&sessions_mutex);
 			JANUS_LOG(LOG_ERR, "No session associated with this handle...\n");
@@ -1125,13 +1361,42 @@ static void *janus_recordplay_handler(void *data) {
 			rec->completed = FALSE;
 			rec->offer = NULL;
 			janus_mutex_init(&rec->mutex);
+			/* Check which codec we should record for audio and/or video */
+			gboolean audio = FALSE, video = FALSE;
+			janus_sdp_find_preferred_codecs(offer, &rec->acodec, &rec->vcodec);
+			/* We found preferred codecs: let's just make sure the direction is what we need */
+			janus_sdp_mline *m = janus_sdp_mline_find(offer, JANUS_SDP_AUDIO);
+			if(m != NULL && m->direction == JANUS_SDP_RECVONLY)
+				rec->acodec = NULL;
+			audio = (rec->acodec != NULL);
+			if(audio) {
+				JANUS_LOG(LOG_WARN, "Audio codec: %s\n", rec->acodec);
+			}
+			m = janus_sdp_mline_find(offer, JANUS_SDP_VIDEO);
+			if(m != NULL && m->direction == JANUS_SDP_RECVONLY)
+				rec->vcodec = NULL;
+			video = (rec->vcodec != NULL);
+			if(video) {
+				JANUS_LOG(LOG_WARN, "Video codec: %s\n", rec->acodec);
+			}
+			rec->audio_pt = AUDIO_PT;
+			if(rec->acodec) {
+				/* Some audio codecs have a fixed payload type that we can't mess with */
+				if(!strcasecmp(rec->acodec, "pcmu"))
+					rec->audio_pt = 0;
+				else if(!strcasecmp(rec->acodec, "pcma"))
+					rec->audio_pt = 8;
+				else if(!strcasecmp(rec->acodec, "g722"))
+					rec->audio_pt = 9;
+			}
+			rec->video_pt = VIDEO_PT;
 			/* Create a date string */
 			time_t t = time(NULL);
 			struct tm *tmv = localtime(&t);
 			char outstr[200];
 			strftime(outstr, sizeof(outstr), "%Y-%m-%d %H:%M:%S", tmv);
 			rec->date = g_strdup(outstr);
-			if(strstr(msg_sdp, "m=audio")) {
+			if(audio) {
 				char filename[256];
 				if(filename_text != NULL) {
 					g_snprintf(filename, 256, "%s-audio", filename_text);
@@ -1139,10 +1404,9 @@ static void *janus_recordplay_handler(void *data) {
 					g_snprintf(filename, 256, "rec-%"SCNu64"-audio", id);
 				}
 				rec->arc_file = g_strdup(filename);
-				/* FIXME Assuming Opus */
-				session->arc = janus_recorder_create(recordings_path, "opus", rec->arc_file);
+				session->arc = janus_recorder_create(recordings_path, rec->acodec, rec->arc_file);
 			}
-			if(strstr(msg_sdp, "m=video")) {
+			if(video) {
 				char filename[256];
 				if(filename_text != NULL) {
 					g_snprintf(filename, 256, "%s-video", filename_text);
@@ -1150,8 +1414,7 @@ static void *janus_recordplay_handler(void *data) {
 					g_snprintf(filename, 256, "rec-%"SCNu64"-video", id);
 				}
 				rec->vrc_file = g_strdup(filename);
-				/* FIXME Assuming VP8 */
-				session->vrc = janus_recorder_create(recordings_path, "vp8", rec->vrc_file);
+				session->vrc = janus_recorder_create(recordings_path, rec->vcodec, rec->vrc_file);
 			}
 			session->recorder = TRUE;
 			session->recording = rec;
@@ -1160,9 +1423,11 @@ static void *janus_recordplay_handler(void *data) {
 			janus_mutex_unlock(&recordings_mutex);
 			/* We need to prepare an answer */
 			janus_sdp *answer = janus_sdp_generate_answer(offer,
-				JANUS_SDP_OA_AUDIO_CODEC, "opus",
+				JANUS_SDP_OA_AUDIO, audio,
+				JANUS_SDP_OA_AUDIO_CODEC, rec->acodec,
 				JANUS_SDP_OA_AUDIO_DIRECTION, JANUS_SDP_RECVONLY,
-				JANUS_SDP_OA_VIDEO_CODEC, "vp8",
+				JANUS_SDP_OA_VIDEO, video,
+				JANUS_SDP_OA_VIDEO_CODEC, rec->vcodec,
 				JANUS_SDP_OA_VIDEO_DIRECTION, JANUS_SDP_RECVONLY,
 				JANUS_SDP_OA_DATA, FALSE,
 				JANUS_SDP_OA_DONE);
@@ -1174,6 +1439,12 @@ static void *janus_recordplay_handler(void *data) {
 			janus_sdp_free(offer);
 			janus_sdp_free(answer);
 			JANUS_LOG(LOG_VERB, "Going to answer this SDP:\n%s\n", sdp);
+			/* If the user negotiated simulcasting, just stick with the base substream */
+			json_t *msg_simulcast = json_object_get(msg->jsep, "simulcast");
+			if(msg_simulcast) {
+				JANUS_LOG(LOG_WARN, "Recording client negotiated simulcasting which we don't do here, falling back to base substream...\n");
+				session->simulcast_ssrc = json_integer_value(json_object_get(msg_simulcast, "ssrc-0"));
+			}
 			/* Done! */
 			result = json_object();
 			json_object_set_new(result, "status", json_string("recording"));
@@ -1280,65 +1551,8 @@ static void *janus_recordplay_handler(void *data) {
 			}
 		} else if(!strcasecmp(request_text, "stop")) {
 			/* Stop the recording/playout */
-			session->active = FALSE;
-			janus_mutex_lock(&session->rec_mutex);
-			if(session->arc) {
-				janus_recorder_close(session->arc);
-				JANUS_LOG(LOG_INFO, "Closed audio recording %s\n", session->arc->filename ? session->arc->filename : "??");
-				janus_recorder_free(session->arc);
-			}
-			session->arc = NULL;
-			if(session->vrc) {
-				janus_recorder_close(session->vrc);
-				JANUS_LOG(LOG_INFO, "Closed video recording %s\n", session->vrc->filename ? session->vrc->filename : "??");
-				janus_recorder_free(session->vrc);
-			}
-			session->vrc = NULL;
-			janus_mutex_unlock(&session->rec_mutex);
-			if(session->recorder) {
-				/* Create a .nfo file for this recording */
-				char nfofile[1024], nfo[1024];
-				g_snprintf(nfofile, 1024, "%s/%"SCNu64".nfo", recordings_path, session->recording->id);
-				FILE *file = fopen(nfofile, "wt");
-				if(file == NULL) {
-					JANUS_LOG(LOG_ERR, "Error creating file %s...\n", nfofile);
-				} else {
-					if(session->recording->arc_file && session->recording->vrc_file) {
-						g_snprintf(nfo, 1024,
-							"[%"SCNu64"]\r\n"
-							"name = %s\r\n"
-							"date = %s\r\n"
-							"audio = %s.mjr\r\n"
-							"video = %s.mjr\r\n",
-								session->recording->id, session->recording->name, session->recording->date,
-								session->recording->arc_file, session->recording->vrc_file);
-					} else if(session->recording->arc_file) {
-						g_snprintf(nfo, 1024,
-							"[%"SCNu64"]\r\n"
-							"name = %s\r\n"
-							"date = %s\r\n"
-							"audio = %s.mjr\r\n",
-								session->recording->id, session->recording->name, session->recording->date,
-								session->recording->arc_file);
-					} else if(session->recording->vrc_file) {
-						g_snprintf(nfo, 1024,
-							"[%"SCNu64"]\r\n"
-							"name = %s\r\n"
-							"date = %s\r\n"
-							"video = %s.mjr\r\n",
-								session->recording->id, session->recording->name, session->recording->date,
-								session->recording->vrc_file);
-					}
-					/* Write to the file now */
-					fwrite(nfo, strlen(nfo), sizeof(char), file);
-					fclose(file);
-					/* Generate the offer */
-					if(janus_recordplay_generate_offer(session->recording) < 0) {
-						JANUS_LOG(LOG_WARN, "Could not generate offer for recording %"SCNu64"...\n", session->recording->id);
-					}
-					session->recording->completed = TRUE;
-				}
-			}
+			janus_recordplay_hangup_media(session->handle);
+
 			/* Done! */
 			result = json_object();
 			json_object_set_new(result, "status", json_string("stopped"));
@@ -1493,13 +1707,28 @@ void janus_recordplay_update_recordings_list(void) {
 			char *ext = strstr(rec->arc_file, ".mjr");
 			if(ext != NULL)
 				*ext = '\0';
+			/* Check which codec is in this recording */
+			rec->acodec = janus_recordplay_parse_codec(recordings_path, rec->arc_file);
 		}
 		if(video && video->value) {
 			rec->vrc_file = g_strdup(video->value);
 			char *ext = strstr(rec->vrc_file, ".mjr");
 			if(ext != NULL)
 				*ext = '\0';
+			/* Check which codec is in this recording */
+			rec->vcodec = janus_recordplay_parse_codec(recordings_path, rec->vrc_file);
 		}
+		rec->audio_pt = AUDIO_PT;
+		if(rec->acodec) {
+			/* Some audio codecs have a fixed payload type that we can't mess with */
+			if(!strcasecmp(rec->acodec, "pcmu"))
+				rec->audio_pt = 0;
+			else if(!strcasecmp(rec->acodec, "pcma"))
+				rec->audio_pt = 8;
+			else if(!strcasecmp(rec->acodec, "g722"))
+				rec->audio_pt = 9;
+		}
+		rec->video_pt = VIDEO_PT;
 		rec->viewers = NULL;
 		rec->destroyed = 0;
 		rec->completed = TRUE;
@@ -1597,9 +1826,9 @@ janus_recordplay_frame_packet *janus_recordplay_get_frames(const char *dir, cons
 				JANUS_LOG(LOG_VERB, "Old .mjr header format\n");
 				bytes = fread(prebuffer, sizeof(char), 5, file);
 				if(prebuffer[0] == 'v') {
-					JANUS_LOG(LOG_INFO, "This is a video recording, assuming VP8\n");
+					JANUS_LOG(LOG_INFO, "This is an old video recording, assuming VP8\n");
 				} else if(prebuffer[0] == 'a') {
-					JANUS_LOG(LOG_INFO, "This is an audio recording, assuming Opus\n");
+					JANUS_LOG(LOG_INFO, "This is an old audio recording, assuming Opus\n");
 				} else {
 					JANUS_LOG(LOG_WARN, "Unsupported recording media type...\n");
 					fclose(file);
@@ -1668,17 +1897,6 @@ janus_recordplay_frame_packet *janus_recordplay_get_frames(const char *dir, cons
 					return NULL;
 				}
 				const char *c = json_string_value(codec);
-				if(video && strcasecmp(c, "vp8")) {
-					JANUS_LOG(LOG_WARN, "The Record&Play plugin only supports VP8 video for now (was '%s')...\n", c);
-					json_decref(info);
-					fclose(file);
-					return NULL;
-				} else if(!video && strcasecmp(c, "opus")) {
-					JANUS_LOG(LOG_WARN, "The Record&Play plugin only supports Opus audio for now (was '%s')...\n", c);
-					json_decref(info);
-					fclose(file);
-					return NULL;
-				}
 				/* When was the file created? */
 				json_t *created = json_object_get(info, "s");
 				if(!created || !json_is_integer(created)) {
@@ -1711,7 +1929,7 @@ janus_recordplay_frame_packet *janus_recordplay_get_frames(const char *dir, cons
 		}
 		/* Only read RTP header */
 		bytes = fread(prebuffer, sizeof(char), 16, file);
-		rtp_header *rtp = (rtp_header *)prebuffer;
+		janus_rtp_header *rtp = (janus_rtp_header *)prebuffer;
 		if(last_ts == 0) {
 			first_ts = ntohl(rtp->timestamp);
 			if(first_ts > 1000*1000)	/* Just used to check whether a packet is pre- or post-reset */
@@ -1758,7 +1976,7 @@ janus_recordplay_frame_packet *janus_recordplay_get_frames(const char *dir, cons
 			JANUS_LOG(LOG_WARN, "Error reading RTP header, stopping here...\n");
 			break;
 		}
-		rtp_header *rtp = (rtp_header *)prebuffer;
+		janus_rtp_header *rtp = (janus_rtp_header *)prebuffer;
 		JANUS_LOG(LOG_HUGE, "  -- RTP packet (ssrc=%"SCNu32", pt=%"SCNu16", ext=%"SCNu16", seq=%"SCNu16", ts=%"SCNu32")\n",
 				ntohl(rtp->ssrc), rtp->type, rtp->extension, ntohs(rtp->seq_number), ntohl(rtp->timestamp));
 		/* Generate frame packet and insert in the ordered list */
@@ -1931,7 +2149,15 @@ static void *janus_recordplay_playout_thread(void *data) {
 	memset(buffer, 0, 1500);
 	int bytes = 0;
 	int64_t ts_diff = 0, passed = 0;
-	
+
+	int audio_pt = session->recording->audio_pt;
+	int video_pt = session->recording->video_pt;
+
+	int akhz = 48;
+	if(audio_pt == 0 || audio_pt == 8 || audio_pt == 9)
+		akhz = 8;
+	int vkhz = 90;
+
 	while(!session->destroyed && session->active && !session->recording->destroyed && (audio || video)) {
 		if(!asent && !vsent) {
 			/* We skipped the last round, so sleep a bit (5ms) */
@@ -1947,8 +2173,8 @@ static void *janus_recordplay_playout_thread(void *data) {
 				if(bytes != audio->len)
 					JANUS_LOG(LOG_WARN, "Didn't manage to read all the bytes we needed (%d < %d)...\n", bytes, audio->len);
 				/* Update payload type */
-				rtp_header *rtp = (rtp_header *)buffer;
-				rtp->type = OPUS_PT;	/* FIXME We assume it's Opus */
+				janus_rtp_header *rtp = (janus_rtp_header *)buffer;
+				rtp->type = audio_pt;
 				if(gateway != NULL)
 					gateway->relay_rtp(session->handle, 0, (char *)buffer, bytes);
 				gettimeofday(&now, NULL);
@@ -1959,7 +2185,7 @@ static void *janus_recordplay_playout_thread(void *data) {
 			} else {
 				/* What's the timestamp skip from the previous packet? */
 				ts_diff = audio->ts - audio->prev->ts;
-				ts_diff = (ts_diff*1000)/48;	/* FIXME Again, we're assuming Opus and it's 48khz */
+				ts_diff = (ts_diff*1000)/akhz;
 				/* Check if it's time to send */
 				gettimeofday(&now, NULL);
 				d_s = now.tv_sec - abefore.tv_sec;
@@ -1988,8 +2214,8 @@ static void *janus_recordplay_playout_thread(void *data) {
 					if(bytes != audio->len)
 						JANUS_LOG(LOG_WARN, "Didn't manage to read all the bytes we needed (%d < %d)...\n", bytes, audio->len);
 					/* Update payload type */
-					rtp_header *rtp = (rtp_header *)buffer;
-					rtp->type = OPUS_PT;	/* FIXME We assume it's Opus */
+					janus_rtp_header *rtp = (janus_rtp_header *)buffer;
+					rtp->type = audio_pt;
 					if(gateway != NULL)
 						gateway->relay_rtp(session->handle, 0, (char *)buffer, bytes);
 					asent = TRUE;
@@ -2007,8 +2233,8 @@ static void *janus_recordplay_playout_thread(void *data) {
 					if(bytes != video->len)
 						JANUS_LOG(LOG_WARN, "Didn't manage to read all the bytes we needed (%d < %d)...\n", bytes, video->len);
 					/* Update payload type */
-					rtp_header *rtp = (rtp_header *)buffer;
-					rtp->type = VP8_PT;	/* FIXME We assume it's VP8 */
+					janus_rtp_header *rtp = (janus_rtp_header *)buffer;
+					rtp->type = video_pt;
 					if(gateway != NULL)
 						gateway->relay_rtp(session->handle, 1, (char *)buffer, bytes);
 					video = video->next;
@@ -2020,7 +2246,7 @@ static void *janus_recordplay_playout_thread(void *data) {
 			} else {
 				/* What's the timestamp skip from the previous packet? */
 				ts_diff = video->ts - video->prev->ts;
-				ts_diff = (ts_diff*1000)/90;
+				ts_diff = (ts_diff*1000)/vkhz;
 				/* Check if it's time to send */
 				gettimeofday(&now, NULL);
 				d_s = now.tv_sec - vbefore.tv_sec;
@@ -2052,8 +2278,8 @@ static void *janus_recordplay_playout_thread(void *data) {
 						if(bytes != video->len)
 							JANUS_LOG(LOG_WARN, "Didn't manage to read all the bytes we needed (%d < %d)...\n", bytes, video->len);
 						/* Update payload type */
-						rtp_header *rtp = (rtp_header *)buffer;
-						rtp->type = VP8_PT;	/* FIXME We assume it's VP8 */
+						janus_rtp_header *rtp = (janus_rtp_header *)buffer;
+						rtp->type = video_pt;
 						if(gateway != NULL)
 							gateway->relay_rtp(session->handle, 1, (char *)buffer, bytes);
 						video = video->next;
