@@ -162,8 +162,8 @@ int janus_rtcp_fix_ssrc(rtcp_context *ctx, char *packet, int len, int fixssrc, u
 			case RTCP_SR: {
 				/* SR, sender report */
 				JANUS_LOG(LOG_HUGE, "     #%d SR (200)\n", pno);
-				rtcp_sr *sr = (rtcp_sr*)rtcp;
-				/* RTCP context provided, update it with info on this SR */
+				rtcp_sr *sr = (rtcp_sr *)rtcp;
+				/* If an RTCP context was provided, update it with info on this SR */
 				janus_rtcp_incoming_sr(ctx, sr);
 				if(fixssrc && newssrcl) {
 					sr->ssrc = htonl(newssrcl);
@@ -176,11 +176,9 @@ int janus_rtcp_fix_ssrc(rtcp_context *ctx, char *packet, int len, int fixssrc, u
 			case RTCP_RR: {
 				/* RR, receiver report */
 				JANUS_LOG(LOG_HUGE, "     #%d RR (201)\n", pno);
-				rtcp_rr *rr = (rtcp_rr*)rtcp;
-				if(ctx != NULL) {
-					/* RTCP context provided, update it with info on this SR */
-					janus_rtcp_incoming_rr(ctx, rr);
-				}
+				rtcp_rr *rr = (rtcp_rr *)rtcp;
+				/* If an RTCP context was provided, update it with info on this RR */
+				janus_rtcp_incoming_rr(ctx, rr);
 				if(fixssrc && newssrcl) {
 					rr->ssrc = htonl(newssrcl);
 				}
@@ -192,7 +190,7 @@ int janus_rtcp_fix_ssrc(rtcp_context *ctx, char *packet, int len, int fixssrc, u
 			case RTCP_SDES: {
 				/* SDES, source description */
 				JANUS_LOG(LOG_HUGE, "     #%d SDES (202)\n", pno);
-				rtcp_sdes *sdes = (rtcp_sdes*)rtcp;
+				rtcp_sdes *sdes = (rtcp_sdes *)rtcp;
 				//~ JANUS_LOG(LOG_HUGE, "       -- SSRC: %u\n", ntohl(sdes->chunk.ssrc));
 				if(fixssrc && newssrcl) {
 					sdes->chunk.ssrc = htonl(newssrcl);
@@ -202,7 +200,7 @@ int janus_rtcp_fix_ssrc(rtcp_context *ctx, char *packet, int len, int fixssrc, u
 			case RTCP_BYE: {
 				/* BYE, goodbye */
 				JANUS_LOG(LOG_HUGE, "     #%d BYE (203)\n", pno);
-				rtcp_bye_t *bye = (rtcp_bye_t*)rtcp;
+				rtcp_bye_t *bye = (rtcp_bye_t *)rtcp;
 				//~ JANUS_LOG(LOG_HUGE, "       -- SSRC: %u\n", ntohl(bye->ssrc[0]));
 				if(fixssrc && newssrcl) {
 					bye->ssrc[0] = htonl(newssrcl);
@@ -212,7 +210,7 @@ int janus_rtcp_fix_ssrc(rtcp_context *ctx, char *packet, int len, int fixssrc, u
 			case RTCP_APP: {
 				/* APP, application-defined */
 				JANUS_LOG(LOG_HUGE, "     #%d APP (204)\n", pno);
-				rtcp_app_t *app = (rtcp_app_t*)rtcp;
+				rtcp_app_t *app = (rtcp_app_t *)rtcp;
 				//~ JANUS_LOG(LOG_HUGE, "       -- SSRC: %u\n", ntohl(app->ssrc));
 				if(fixssrc && newssrcl) {
 					app->ssrc = htonl(newssrcl);
@@ -541,7 +539,7 @@ int janus_rtcp_report_block(rtcp_context *ctx, report_block *rb) {
 	rb->flcnpl = htonl(lost | fraction);
 	if(ctx->lsr > 0) {
 		rb->lsr = htonl(ctx->lsr);
-		rb->delay = htonl(((now - ctx->lsr_ts) / 1000000) << 16);
+		rb->delay = htonl(((now - ctx->lsr_ts) << 16) / 1000000);
 	} else {
 		rb->lsr = 0;
 		rb->delay = 0;
@@ -873,7 +871,15 @@ int janus_rtcp_sdes(char *packet, int len, const char *cname, int cnamelen) {
 
 /* Generate a new REMB message */
 int janus_rtcp_remb(char *packet, int len, uint32_t bitrate) {
-	if(packet == NULL || len != 24)
+	/* By default we assume a single SSRC will be set */
+	return janus_rtcp_remb_ssrcs(packet, len, bitrate, 1);
+}
+
+int janus_rtcp_remb_ssrcs(char *packet, int len, uint32_t bitrate, uint8_t numssrc) {
+	if(packet == NULL || numssrc == 0)
+		return -1;
+	int min_len = 20 + numssrc*4;
+	if(len < min_len)
 		return -1;
 	memset(packet, 0, len);
 	rtcp_header *rtcp = (rtcp_header *)packet;
@@ -881,7 +887,7 @@ int janus_rtcp_remb(char *packet, int len, uint32_t bitrate) {
 	rtcp->version = 2;
 	rtcp->type = RTCP_PSFB;
 	rtcp->rc = 15;
-	rtcp->length = htons((len/4)-1);
+	rtcp->length = htons((min_len/4)-1);
 	/* Now set REMB stuff */
 	rtcp_fb *rtcpfb = (rtcp_fb *)rtcp;
 	rtcp_remb *remb = (rtcp_remb *)rtcpfb->fci;
@@ -905,12 +911,12 @@ int janus_rtcp_remb(char *packet, int len, uint32_t bitrate) {
 	/* FIXME From rtcp_sender.cc */
 	unsigned char *_ptrRTCPData = (unsigned char *)remb;
 	_ptrRTCPData += 4;	/* Skip unique identifier */
-	_ptrRTCPData[0] = (uint8_t)(1);	/* Just one SSRC */
+	_ptrRTCPData[0] = numssrc;
 	_ptrRTCPData[1] = (uint8_t)((newbrexp << 2) + ((newbrmantissa >> 16) & 0x03));
 	_ptrRTCPData[2] = (uint8_t)(newbrmantissa >> 8);
 	_ptrRTCPData[3] = (uint8_t)(newbrmantissa);
 	JANUS_LOG(LOG_HUGE, "[REMB] bitrate=%"SCNu32" (%d bytes)\n", bitrate, 4*(ntohs(rtcp->length)+1));
-	return 24;
+	return min_len;
 }
 
 /* Generate a new FIR message */
