@@ -278,6 +278,30 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp) {
 				g_free(stream->rpass);
 			stream->rpass = g_strdup(rpass);
 		}
+		/* Is simulcasting enabled, using rid? (we need to check this before parsing SSRCs) */
+		tempA = m->attributes;
+		while(tempA) {
+			janus_sdp_attribute *a = (janus_sdp_attribute *)tempA->data;
+			if(a->name && !strcasecmp(a->name, "rid")) {
+				/* This attribute is used by Firefox for simulcasting */
+				char rid[16];
+				if(sscanf(a->value, "%15s send", rid) != 1) {
+					JANUS_LOG(LOG_ERR, "[%"SCNu64"] Failed to parse rid attribute...\n", handle->handle_id);
+				} else {
+					JANUS_LOG(LOG_VERB, "[%"SCNu64"] Parsed rid: %s\n", handle->handle_id, rid);
+					if(stream->rid[0] == NULL) {
+						stream->rid[0] = g_strdup(rid);
+					} else if(stream->rid[1] == NULL) {
+						stream->rid[1] = g_strdup(rid);
+					} else if(stream->rid[2] == NULL) {
+						stream->rid[2] = g_strdup(rid);
+					} else {
+						JANUS_LOG(LOG_WARN, "[%"SCNu64"] Too many RTP Stream IDs, ignoring '%s'...\n", handle->handle_id, rid);
+					}
+				}
+			}
+			tempA = tempA->next;
+		}
 		/* Now look for candidates and other info */
 		tempA = m->attributes;
 		while(tempA) {
@@ -296,15 +320,30 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp) {
 							JANUS_LOG(LOG_ERR, "[%"SCNu64"] Failed to parse candidate... (%d)\n", handle->handle_id, res);
 						}
 					}
-				}
-				if(!strcasecmp(a->name, "ssrc")) {
+				} else if(!strcasecmp(a->name, "ssrc-group")) {
+					/* FIXME This can be either FID or SIM */
+					int res = janus_sdp_parse_ssrc_group(stream, (const char *)a->value, m->type == JANUS_SDP_VIDEO);
+					if(res != 0) {
+						JANUS_LOG(LOG_ERR, "[%"SCNu64"] Failed to parse SSRC group attribute... (%d)\n", handle->handle_id, res);
+					}
+				} else if(!strcasecmp(a->name, "ssrc")) {
 					int res = janus_sdp_parse_ssrc(stream, (const char *)a->value, m->type == JANUS_SDP_VIDEO);
 					if(res != 0) {
 						JANUS_LOG(LOG_ERR, "[%"SCNu64"] Failed to parse SSRC attribute... (%d)\n", handle->handle_id, res);
 					}
+				} else if(!strcasecmp(a->name, "rtcp-fb")) {
+					if(a->value && strstr(a->value, "nack") && stream->rtp_component) {
+						if(m->type == JANUS_SDP_AUDIO) {
+							/* Enable NACKs for audio */
+							stream->rtp_component->do_audio_nacks = TRUE;
+						} else if(m->type == JANUS_SDP_VIDEO) {
+							/* Enable NACKs for video */
+							stream->rtp_component->do_video_nacks = TRUE;
+						}
+					}
 				}
 #ifdef HAVE_SCTP
-				if(!strcasecmp(a->name, "sctpmap")) {
+				else if(!strcasecmp(a->name, "sctpmap")) {
 					/* TODO Parse sctpmap line to get the UDP-port value and the number of channels */
 					JANUS_LOG(LOG_VERB, "Got a sctpmap attribute: %s\n", a->value);
 				}
@@ -312,6 +351,14 @@ int janus_sdp_process(void *ice_handle, janus_sdp *remote_sdp) {
 			}
 			tempA = tempA->next;
 		}
+		if(stream->video_ssrc_peer[1] && stream->video_rtcp_ctx[1] == NULL) {
+			stream->video_rtcp_ctx[1] = g_malloc0(sizeof(rtcp_context));
+			stream->video_rtcp_ctx[1]->tb = 90000;
+		}
+		if(stream->video_ssrc_peer[2] && stream->video_rtcp_ctx[2] == NULL) {
+			stream->video_rtcp_ctx[2] = g_malloc0(sizeof(rtcp_context));
+			stream->video_rtcp_ctx[2]->tb = 90000;
+		}
 		temp = temp->next;
 	}
 	if(ruser)
@@ -545,6 +592,64 @@ int janus_sdp_parse_candidate(void *ice_stream, const char *candidate, int trick
 	return 0;
 }
 
+int janus_sdp_parse_ssrc_group(void *ice_stream, const char *group_attr, int video) {
+	if(ice_stream == NULL || group_attr == NULL)
+		return -1;
+	janus_ice_stream *stream = (janus_ice_stream *)ice_stream;
+	janus_ice_handle *handle = stream->handle;
+	if(handle == NULL)
+		return -2;
+	if(!video)
+		return -3;
+	gboolean fid = strstr(group_attr, "FID") != NULL;
+	gboolean sim = strstr(group_attr, "SIM") != NULL;
+	guint64 ssrc = 0;
+	gchar **list = g_strsplit(group_attr, " ", -1);
+	gchar *index = list[0];
+	if(index != NULL) {
+		int i=0;
+		while(index != NULL) {
+			if(i > 0 && strlen(index) > 0) {
+				ssrc = g_ascii_strtoull(index, NULL, 0);
+				switch(i) {
+					case 1:
+						stream->video_ssrc_peer[0] = ssrc;
+						JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC: %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer[0]);
+						break;
+					case 2:
+						if(fid) {
+							stream->video_ssrc_peer_rtx = ssrc;
+							JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (rtx): %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer_rtx);
+						} else if(sim) {
+							stream->video_ssrc_peer[1] = ssrc;
+							JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (sim-1): %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer[1]);
+						} else {
+							JANUS_LOG(LOG_WARN, "[%"SCNu64"] Don't know what to do with SSRC: %"SCNu64"\n", handle->handle_id, ssrc);
+						}
+						break;
+					case 3:
+						if(fid) {
+							JANUS_LOG(LOG_WARN, "[%"SCNu64"] Found one too many retransmission SSRC (rtx): %"SCNu64"\n", handle->handle_id, ssrc);
+						} else if(sim) {
+							stream->video_ssrc_peer[2] = ssrc;
+							JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (sim-2): %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer[2]);
+						} else {
+							JANUS_LOG(LOG_WARN, "[%"SCNu64"] Don't know what to do with SSRC: %"SCNu64"\n", handle->handle_id, ssrc);
+						}
+						break;
+					default:
+						JANUS_LOG(LOG_WARN, "[%"SCNu64"] Don't know what to do with video SSRC: %"SCNu64"\n", handle->handle_id, ssrc);
+						break;
+				}
+			}
+			i++;
+			index = list[i];
+		}
+	}
+	g_clear_pointer(&list, g_strfreev);
+	return 0;
+}
+
 int janus_sdp_parse_ssrc(void *ice_stream, const char *ssrc_attr, int video) {
 	if(ice_stream == NULL || ssrc_attr == NULL)
 		return -1;
@@ -556,20 +661,27 @@ int janus_sdp_parse_ssrc(void *ice_stream, const char *ssrc_attr, int video) {
 	if(ssrc == 0 || ssrc > G_MAXUINT32)
 		return -3;
 	if(video) {
-		if(stream->video_ssrc_peer == 0) {
-			stream->video_ssrc_peer = ssrc;
-			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC: %u\n", handle->handle_id, stream->video_ssrc_peer);
-		} else if(stream->video_ssrc_peer != ssrc) {
-			/* FIXME We assume the second SSRC we get is the one Chrome associates with retransmissions, e.g.
-			 * 	a=ssrc-group:FID 586466331 2053167359 (SSRC SSRC-rtx)
-			 * SSRC group FID: https://tools.ietf.org/html/rfc3388#section-7 */
-			stream->video_ssrc_peer_rtx = ssrc;
-			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (rtx): %u\n", handle->handle_id, stream->video_ssrc_peer_rtx);
+		if(stream->video_ssrc_peer[0] == 0) {
+			stream->video_ssrc_peer[0] = ssrc;
+			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC: %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer[0]);
+		} else {
+			/* We already have a video SSRC: check if RID is involved, and we'll keep track of this for simulcasting */
+			if(stream->rid[0]) {
+				if(stream->video_ssrc_peer[1] == 0) {
+					stream->video_ssrc_peer[1] = ssrc;
+					JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (sim-1): %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer[1]);
+				} else if(stream->video_ssrc_peer[2] == 0) {
+					stream->video_ssrc_peer[2] = ssrc;
+					JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer video SSRC (sim-2): %"SCNu32"\n", handle->handle_id, stream->video_ssrc_peer[2]);
+				} else {
+					JANUS_LOG(LOG_WARN, "[%"SCNu64"] Don't know what to do with video SSRC: %"SCNu64"\n", handle->handle_id, ssrc);
+				}
+			}
 		}
 	} else {
 		if(stream->audio_ssrc_peer == 0) {
 			stream->audio_ssrc_peer = ssrc;
-			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer audio SSRC: %u\n", handle->handle_id, stream->audio_ssrc_peer);
+			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Peer audio SSRC: %"SCNu32"\n", handle->handle_id, stream->audio_ssrc_peer);
 		}
 	}
 	return 0;
@@ -647,6 +759,7 @@ int janus_sdp_anonymize(janus_sdp *anon) {
 					|| !strcasecmp(a->name, "mid")
 					|| !strcasecmp(a->name, "msid")
 					|| !strcasecmp(a->name, "msid-semantic")
+					|| !strcasecmp(a->name, "rid")
 					|| !strcasecmp(a->name, "rtcp")
 					|| !strcasecmp(a->name, "rtcp-mux")
 					|| !strcasecmp(a->name, "rtcp-rsize")
@@ -927,11 +1040,36 @@ char *janus_sdp_merge(void *ice_handle, janus_sdp *anon) {
 			a = janus_sdp_attribute_create("ssrc", "%"SCNu32" label:janusv0", stream->video_ssrc);
 			m->attributes = g_list_append(m->attributes, a);
 		}
+		/* FIXME If the peer is Firefox and is negotiating simulcasting, add the rid attributes */
+		if(m->type == JANUS_SDP_VIDEO && stream->rid[0] != NULL) {
+			char rids[50];
+			rids[0] = '\0';
+			int i=0;
+			for(i=0; i<3; i++) {
+				if(stream->rid[i] == NULL)
+					continue;
+				a = janus_sdp_attribute_create("rid", "%s recv", stream->rid[i]);
+				m->attributes = g_list_append(m->attributes, a);
+				if(strlen(rids) == 0) {
+					g_strlcat(rids, stream->rid[i], sizeof(rids));
+				} else {
+					g_strlcat(rids, ";", sizeof(rids));
+					g_strlcat(rids, stream->rid[i], sizeof(rids));
+				}
+			}
+			a = janus_sdp_attribute_create("simulcast", " recv rid=%s", rids);
+			m->attributes = g_list_append(m->attributes, a);
+		}
 		/* And now the candidates */
 		janus_ice_candidates_to_sdp(handle, m, stream->stream_id, 1);
 		if(!janus_flags_is_set(&handle->webrtc_flags, JANUS_ICE_HANDLE_WEBRTC_RTCPMUX) &&
 				(m->type == JANUS_SDP_AUDIO || m->type == JANUS_SDP_VIDEO))
 			janus_ice_candidates_to_sdp(handle, m, stream->stream_id, 2);
+		/* Since we're half-trickling, we need to notify the peer that these are all the
+		 * candidates we have for this media stream, via an end-of-candidates attribute:
+		 * https://tools.ietf.org/html/draft-ietf-mmusic-trickle-ice-02#section-4.1 */
+		janus_sdp_attribute *end = janus_sdp_attribute_create("end-of-candidates", NULL);
+		m->attributes = g_list_append(m->attributes, end);
 		/* Next */
 		temp = temp->next;
 	}
