@@ -129,7 +129,7 @@ typedef struct janus_pfunix_client {
 	gboolean session_timeout;	/* Whether a Janus session timeout occurred in the core */
 } janus_pfunix_client;
 static GHashTable *clients = NULL, *clients_by_fd = NULL, *clients_by_path = NULL;
-static janus_mutex clients_mutex;
+static janus_mutex clients_mutex = JANUS_MUTEX_INITIALIZER;
 
 
 /* Helper to create a named Unix Socket out of the path to link to */
@@ -289,7 +289,6 @@ int janus_pfunix_init(janus_transport_callbacks *callback, const char *config_pa
 	clients = g_hash_table_new(NULL, NULL);
 	clients_by_fd = g_hash_table_new(NULL, NULL);
 	clients_by_path = g_hash_table_new(g_str_hash, g_str_equal);
-	janus_mutex_init(&clients_mutex);
 
 	/* Start the Unix Sockets service thread */
 	GError *error = NULL;
@@ -417,22 +416,11 @@ void janus_pfunix_session_over(void *transport, guint64 session_id, gboolean tim
 	janus_mutex_lock(&clients_mutex);
 	if(g_hash_table_lookup(clients, client) != NULL) {
 		client->session_timeout = TRUE;
-		if(client->fd != -1) {
-			/* Shutdown the client socket */
-			shutdown(client->fd, SHUT_WR);
-		} else {
-			/* Destroy the client */
-			g_hash_table_remove(clients_by_path, client->addr.sun_path);
-			g_hash_table_remove(clients, client);
-			if(client->messages != NULL) {
-				char *response = NULL;
-				while((response = g_async_queue_try_pop(client->messages)) != NULL) {
-					g_free(response);
-				}
-				g_async_queue_unref(client->messages);
-			}
-			g_free(client);
-		}
+		/* Notify the thread about this */
+		int res = 0;
+		do {
+			res = write(write_fd[1], "x", 1);
+		} while(res == -1 && errno == EINTR);
 	}
 	janus_mutex_unlock(&clients_mutex);
 }
@@ -493,7 +481,11 @@ void *janus_pfunix_thread(void *data) {
 		if(res == 0)
 			continue;
 		if(res < 0) {
-			JANUS_LOG(LOG_ERR, "poll() failed\n");
+			if(errno == EINTR) {
+				JANUS_LOG(LOG_HUGE, "Got an EINTR (%s) polling the Unix Sockets descriptors, ignoring...\n", strerror(errno));
+				continue;
+			}
+			JANUS_LOG(LOG_ERR, "poll() failed: %d (%s)\n", errno, strerror(errno));
 			break;
 		}
 		int i = 0;
@@ -573,12 +565,31 @@ void *janus_pfunix_thread(void *data) {
 					while((payload = g_async_queue_try_pop(client->messages)) != NULL) {
 						int res = 0;
 						do {
+							if(client->fd < 0)
+								break;
 							res = write(client->fd, payload, strlen(payload));
 						} while(res == -1 && errno == EINTR);
 						/* FIXME Should we check if sent everything? */
 						JANUS_LOG(LOG_HUGE, "Written %d/%zu bytes on %d\n", res, strlen(payload), client->fd);
 						g_free(payload);
 					}
+					if(client->session_timeout) {
+						/* We should actually get rid of this connection, now */
+						shutdown(SHUT_RDWR, poll_fds[i].fd);
+						close(poll_fds[i].fd);
+						client->fd = -1;
+						/* Destroy the client */
+						g_hash_table_remove(clients_by_fd, GINT_TO_POINTER(poll_fds[i].fd));
+						g_hash_table_remove(clients, client);
+						if(client->messages != NULL) {
+							char *response = NULL;
+							while((response = g_async_queue_try_pop(client->messages)) != NULL) {
+								g_free(response);
+							}
+							g_async_queue_unref(client->messages);
+						}
+						g_free(client);
+					}
 				}
 				janus_mutex_unlock(&clients_mutex);
 			}
